@@ -21,11 +21,11 @@ from SPARQLWrapper import SPARQLWrapper, RDFXML
 
 from oc_ocdm.reader import Reader
 from oc_ocdm.abstract_set import AbstractSet
-from oc_ocdm.support.support import get_count
+from oc_ocdm.support.support import get_count, get_short_name
 
 if TYPE_CHECKING:
-    from typing import Dict, ClassVar, Tuple, Optional, List
-    from rdflib.query import Result
+    from typing import Dict, ClassVar, Tuple, Optional, List, Set
+    from rdflib import ConjunctiveGraph
 
 from rdflib import Graph, Namespace, URIRef
 
@@ -98,105 +98,126 @@ class GraphSet(AbstractSet):
     # Add resources related to bibliographic entities
     def add_an(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> ReferenceAnnotation:
+        if res is not None and get_short_name(res) != "an":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a ReferenceAnnotation entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_an, res, "an")
+        cur_g, count, label = self._add(self.g_an, "an", res)
         return ReferenceAnnotation(cur_g, self, res, GraphEntity.iri_note,
                                    resp_agent, source, count, label, "an",
                                    preexisting_graph)
 
     def add_ar(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> AgentRole:
+        if res is not None and get_short_name(res) != "ar":
+            raise ValueError(f"Given res: <{res}> is inappropriate for an AgentRole entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_ar, res, "ar")
+        cur_g, count, label = self._add(self.g_ar, "ar", res)
         return AgentRole(cur_g, self, res, GraphEntity.iri_role_in_time,
                          resp_agent, source, count, label, "ar",
                          preexisting_graph)
 
     def add_be(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> BibliographicReference:
+        if res is not None and get_short_name(res) != "be":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a BibliographicReference entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_be, res, "be")
-        return BibliographicReference(cur_g, self, res,
-                                      GraphEntity.iri_bibliographic_reference,
+        cur_g, count, label = self._add(self.g_be, "be", res)
+        return BibliographicReference(cur_g, self, res, GraphEntity.iri_bibliographic_reference,
                                       resp_agent, source, count, label, "be",
                                       preexisting_graph)
 
     def add_br(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> BibliographicResource:
+        if res is not None and get_short_name(res) != "br":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a BibliographicResource entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_br, res, "br")
+        cur_g, count, label = self._add(self.g_br, "br", res)
         return BibliographicResource(cur_g, self, res, GraphEntity.iri_expression,
                                      resp_agent, source, count, label, "br",
                                      preexisting_graph)
 
     def add_ci(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> Citation:
+        if res is not None and get_short_name(res) != "ci":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a Citation entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_ci, res, "ci")
+        cur_g, count, label = self._add(self.g_ci, "ci", res)
         return Citation(cur_g, self, res, GraphEntity.iri_citation,
                         resp_agent, source, count, label, "ci",
                         preexisting_graph)
 
     def add_de(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> DiscourseElement:
+        if res is not None and get_short_name(res) != "de":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a DiscourseElement entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_de, res, "de")
+        cur_g, count, label = self._add(self.g_de, "de", res)
         return DiscourseElement(cur_g, self, res, GraphEntity.iri_discourse_element,
                                 resp_agent, source, count, label, "de",
                                 preexisting_graph)
 
     def add_id(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> Identifier:
+        if res is not None and get_short_name(res) != "id":
+            raise ValueError(f"Given res: <{res}> is inappropriate for an Identifier entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_id, res, "id")
+        cur_g, count, label = self._add(self.g_id, "id", res)
         return Identifier(cur_g, self, res, GraphEntity.iri_identifier,
                           resp_agent, source, count, label, "id",
                           preexisting_graph)
 
     def add_pl(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> PointerList:
+        if res is not None and get_short_name(res) != "pl":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a PointerList entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_pl, res, "pl")
+        cur_g, count, label = self._add(self.g_pl, "pl", res)
         return PointerList(cur_g, self, res, GraphEntity.iri_singleloc_pointer_list,
                            resp_agent, source, count, label, "pl",
                            preexisting_graph)
 
     def add_rp(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> ReferencePointer:
+        if res is not None and get_short_name(res) != "rp":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a ReferencePointer entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_rp, res, "rp")
+        cur_g, count, label = self._add(self.g_rp, "rp", res)
         return ReferencePointer(cur_g, self, res, GraphEntity.iri_intextref_pointer,
                                 resp_agent, source, count, label, "rp",
                                 preexisting_graph)
 
     def add_ra(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> ResponsibleAgent:
+        if res is not None and get_short_name(res) != "ra":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a ResponsibleAgent entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_ra, res, "ra")
+        cur_g, count, label = self._add(self.g_ra, "ra", res)
         return ResponsibleAgent(cur_g, self, res, GraphEntity.iri_agent,
                                 resp_agent, source, count, label, "ra",
                                 preexisting_graph)
 
     def add_re(self, resp_agent: str, source: str = None, res: URIRef = None,
                preexisting_graph: Graph = None) -> ResourceEmbodiment:
+        if res is not None and get_short_name(res) != "re":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a ResourceEmbodiment entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add(self.g_re, res, "re")
+        cur_g, count, label = self._add(self.g_re, "re", res)
         return ResourceEmbodiment(cur_g, self, res, GraphEntity.iri_manifestation,
                                   resp_agent, source, count, label, "re",
                                   preexisting_graph)
 
-    def _add(self, graph_url: str, res: URIRef, short_name: str) -> Tuple[Graph, Optional[str], Optional[str]]:
+    def _add(self, graph_url: str, short_name: str, res: URIRef = None) -> Tuple[Graph, Optional[str], Optional[str]]:
         cur_g: Graph = Graph(identifier=graph_url)
         self._set_ns(cur_g)
 
@@ -219,17 +240,34 @@ class GraphSet(AbstractSet):
 
         return cur_g, count, label
 
-    def sync_with_triplestore(self, ts_url: str, resp_agent: str) -> None:
+    def get_orphans(self) -> List[GraphEntity]:
+        full_set_of_entities: Set[URIRef] = set(self.res_to_entity.keys())
+        referenced_entities: Set[URIRef] = set()
+        for res, entity in self.res_to_entity.items():
+            for obj in entity.g.objects(subject=res, predicate=None):
+                if type(obj) == URIRef:
+                    referenced_entities.add(obj)
+        set_of_orphan_res: Set[URIRef] = full_set_of_entities - referenced_entities
+
+        result_list: List[GraphEntity] = []
+        for orphan_res in set_of_orphan_res:
+            entity: Optional[GraphEntity] = self.get_entity(orphan_res)
+            if entity is not None:
+                result_list.append(entity)
+
+        return result_list
+
+    def remove_orphans_from_triplestore(self, ts_url: str, resp_agent: str) -> None:
         sparql: SPARQLWrapper = SPARQLWrapper(ts_url)
 
         for entity_res, entity in self.res_to_entity.items():
             if entity.to_be_deleted:
                 query: str = f"CONSTRUCT {{?s ?p ?o}} WHERE {{?s ?p ?o ; ?p_1 <{entity_res}>}}"
                 sparql.setQuery(query)
-                sparql.setMethod('POST')
+                sparql.setMethod('GET')
                 sparql.setReturnFormat(RDFXML)
 
-                result: Result = sparql.query().convert()
+                result: ConjunctiveGraph = sparql.query().convert()
                 if result is not None:
                     imported_entities: List[GraphEntity] = Reader.import_entities_from_graph(self, result, resp_agent)
                     for imported_entity in imported_entities:
