@@ -22,7 +22,7 @@ from oc_ocdm.counter_handler.filesystem_counter_handler import FilesystemCounter
 from oc_ocdm.counter_handler.in_memory_counter_handler import InMemoryCounterHandler
 from oc_ocdm.metadata.entities.dataset import Dataset
 from oc_ocdm.metadata.entities.distribution import Distribution
-from oc_ocdm.support.support import get_count
+from oc_ocdm.support.support import get_count, is_dataset, get_short_name
 
 if TYPE_CHECKING:
     from typing import Dict, Optional, Tuple, ClassVar
@@ -60,27 +60,31 @@ class MetadataSet(AbstractSet):
 
     def add_dataset(self, dataset_name: str, resp_agent: str, source: str = None, res: URIRef = None,
                     preexisting_graph: Graph = None) -> Dataset:
+        if res is not None and not is_dataset(res):
+            raise ValueError(f"Given res: <{res}> is inappropriate for a Dataset entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
         # Here we use a fictitious short name for Dataset, since the OCDM document doesn't specify
         # any particular short name for this type of entity. It's only used internally to distinguish
         # between different metadata entities but it's meaningless outside of this scope.
-        cur_g, count, label = self._add_metadata(res, "_dataset_", dataset_name)
+        cur_g, count, label = self._add_metadata("_dataset_", dataset_name, res)
         return Dataset(cur_g, self.base_iri, dataset_name, self, res,
                        MetadataEntity.iri_dataset, resp_agent,
                        source, count, label, "_dataset_", preexisting_graph)
 
     def add_di(self, dataset_name: str, resp_agent: str, source: str = None,
                res: URIRef = None, preexisting_graph: Graph = None) -> Distribution:
+        if res is not None and get_short_name(res) != "di":
+            raise ValueError(f"Given res: <{res}> is inappropriate for a Distribution entity.")
         if res is not None and res in self.res_to_entity:
             return self.res_to_entity[res]
-        cur_g, count, label = self._add_metadata(res, "di", dataset_name)
+        cur_g, count, label = self._add_metadata("di", dataset_name, res)
         return Distribution(cur_g, self.base_iri, dataset_name, self, res,
                             MetadataEntity.iri_datafile, resp_agent,
                             source, count, label, "di", preexisting_graph)
 
-    def _add_metadata(self, res: URIRef, short_name: str,
-                      dataset_name: str) -> Tuple[Graph, Optional[str], Optional[str]]:
+    def _add_metadata(self, short_name: str, dataset_name: str,
+                      res: URIRef = None) -> Tuple[Graph, Optional[str], Optional[str]]:
         cur_g: Graph = Graph()
         self._set_ns(cur_g)
 
