@@ -22,6 +22,7 @@ from datetime import datetime
 from rdflib import ConjunctiveGraph
 from SPARQLWrapper import SPARQLWrapper
 from typing import TYPE_CHECKING
+from zipfile import ZipFile
 
 from oc_ocdm.prov.prov_entity import ProvEntity
 from oc_ocdm.graph.graph_entity import GraphEntity
@@ -42,7 +43,7 @@ class Storer(object):
 
     def __init__(self, abstract_set: AbstractSet, repok: Reporter = None, reperr: Reporter = None,
                  context_map: Dict[str, Any] = None, default_dir: str = "_", dir_split: int = 0,
-                 n_file_item: int = 1, output_format: str = "json-ld") -> None:
+                 n_file_item: int = 1, output_format: str = "json-ld", zip_output: bool = False) -> None:
         # We only accept format strings that:
         # 1. are supported by rdflib
         # 2. correspond to an output format which is effectively either NT or NQ
@@ -54,7 +55,7 @@ class Storer(object):
                              f" Available formats: {supported_formats}.")
         else:
             self.output_format: str = output_format
-
+        self.zip_output = zip_output
         self.dir_split: int = dir_split
         self.n_file_item: int = n_file_item
         self.default_dir: str = default_dir if default_dir != "" else "_"
@@ -106,26 +107,36 @@ class Storer(object):
                 break
 
             new_g.addN([(s, p, o, g_iri)])
-
-        if self.output_format == "json-ld":
-            if context_path is not None and context_path in self.context_map:
-                cur_json_ld: Any = json.loads(
-                    new_g.serialize(format="json-ld", context=self.context_map[context_path]))
-
-                if isinstance(cur_json_ld, dict):
-                    cur_json_ld["@context"] = context_path
-                else:  # it is a list
-                    for item in cur_json_ld:
-                        item["@context"] = context_path
+        zip_file_path = cur_file_path.replace(os.path.splitext(cur_file_path)[1], ".zip")
+        lock = FileLock(f"{zip_file_path}.lock") if self.zip_output else FileLock(f"{cur_file_path}.lock")
+        with lock:
+            if self.zip_output:
+                zip_file = ZipFile(zip_file_path, mode="w")
+            if self.output_format == "json-ld":
+                if context_path is not None and context_path in self.context_map:
+                    cur_json_ld: Any = json.loads(
+                        new_g.serialize(format="json-ld", context=self.context_map[context_path]))
+                    if isinstance(cur_json_ld, dict):
+                        cur_json_ld["@context"] = context_path
+                    else:  # it is a list
+                        for item in cur_json_ld:
+                            item["@context"] = context_path
+                else:
+                    cur_json_ld: Any = json.loads(new_g.serialize(format="json-ld"))
+                    if self.zip_output:
+                        dumped_json: bytes = json.dumps(cur_json_ld, ensure_ascii=False).encode('utf-8')
+                        zip_file.writestr(zinfo_or_arcname=os.path.basename(cur_file_path), data=dumped_json)
+                    else:
+                        with open(cur_file_path, 'wt', encoding='utf-8') as f:
+                            json.dump(cur_json_ld, f, ensure_ascii=False)
             else:
-                cur_json_ld: Any = json.loads(new_g.serialize(format="json-ld"))
-            lock = FileLock(f"{cur_file_path}.lock")
-            with lock:
-                with open(cur_file_path, 'wt', encoding='utf-8') as f:
-                    json.dump(cur_json_ld, f, indent=4, ensure_ascii=False)
-        else:
-            new_g.serialize(cur_file_path, format=self.output_format, encoding="utf-8")
-
+                if self.zip_output:
+                    rdf_serialization: bytes = new_g.serialize(destination=None, format=self.output_format, encoding="utf-8")
+                    zip_file.writestr(zinfo_or_arcname=os.path.basename(cur_file_path), data=rdf_serialization)
+                else:
+                    new_g.serialize(destination=cur_file_path, format=self.output_format, encoding="utf-8")
+            if self.zip_output:
+                zip_file.close()
         self.repok.add_sentence(f"File '{cur_file_path}' added.")
 
     def store_all(self, base_dir: str, base_iri: str, context_path: str = None) -> List[str]:
@@ -134,60 +145,49 @@ class Storer(object):
 
         self.repok.add_sentence("Starting the process")
 
-        processed_graphs: Dict[str, ConjunctiveGraph] = {}
+        relevant_paths: Dict[str, list] = dict()
         for entity in self.a_set.res_to_entity.values():
-            processed_graphs = self.store(entity, base_dir, base_iri, context_path,
-                                          processed_graphs, False)
+            cur_dir_path, cur_file_path = self._dir_and_file_paths(entity.res, base_dir, base_iri)
+            if not os.path.exists(cur_dir_path):
+                os.makedirs(cur_dir_path)
+            relevant_paths.setdefault(cur_file_path, list())
+            relevant_paths[cur_file_path].append(entity)
 
-        stored_graph_path: List[str] = []
-        for cur_file_path in processed_graphs:
-            stored_graph_path.append(cur_file_path)
-            self._store_in_file(processed_graphs[cur_file_path], cur_file_path, context_path)
+        for relevant_path, entities_in_path in relevant_paths.items():
+            stored_g = None
+            # Here we try to obtain a reference to the currently stored graph
+            output_filepath = relevant_path.replace(os.path.splitext(relevant_path)[1], ".zip") if self.zip_output else relevant_path
+            if os.path.exists(output_filepath):
+                stored_g = Reader(context_map=self.context_map).load(output_filepath)
+            if stored_g is None:
+                stored_g = ConjunctiveGraph()
+            for entity_in_path in entities_in_path:
+                self.store(entity_in_path, stored_g, relevant_path, context_path, False)
+            self._store_in_file(stored_g, relevant_path, context_path)
 
-        return stored_graph_path
+        return list(relevant_paths.keys())
 
-    def store(self, entity: AbstractEntity, base_dir: str, base_iri: str, context_path: str = None,
-              already_processed: Dict[str, ConjunctiveGraph] = None,
-              store_now: bool = True) -> Optional[Dict[str, ConjunctiveGraph]]:
+    def store(self, entity: AbstractEntity, destination_g: ConjunctiveGraph, cur_file_path: str, context_path: str = None, store_now: bool = True) -> ConjunctiveGraph:
         self.repok.new_article()
         self.reperr.new_article()
 
-        if already_processed is None:
-            already_processed: Dict[str, ConjunctiveGraph] = {}
-
-        cur_dir_path, cur_file_path = self._dir_and_file_paths(entity.res, base_dir, base_iri)
-
         try:
-            if not os.path.exists(cur_dir_path):
-                os.makedirs(cur_dir_path)
-
-            stored_g: Optional[ConjunctiveGraph] = None
-
-            # Here we try to obtain a reference to the currently stored graph
-            if cur_file_path in already_processed:
-                stored_g = already_processed[cur_file_path]
-            elif os.path.exists(cur_file_path):
-                stored_g = Reader(context_map=self.context_map).load(cur_file_path)
-
-            if stored_g is None:
-                stored_g = ConjunctiveGraph()
-
             if isinstance(entity, ProvEntity):
                 quads: List[Tuple] = []
                 graph_identifier: URIRef = entity.g.identifier
                 for triple in entity.g.triples((entity.res, None, None)):
                     quads.append((*triple, graph_identifier))
-                stored_g.addN(quads)
+                destination_g.addN(quads)
             elif isinstance(entity, GraphEntity) or isinstance(entity, MetadataEntity):
                 if entity.to_be_deleted:
-                    stored_g.remove((entity.res, None, None, None))
+                    destination_g.remove((entity.res, None, None, None))
                 else:
                     if len(entity.preexisting_graph) > 0:
                         """
                         We're not in 'append mode', so we need to remove
                         the entity that we're going to overwrite.
                         """
-                        stored_g.remove((entity.res, None, None, None))
+                        destination_g.remove((entity.res, None, None, None))
                     """
                     Here we copy data from the entity into the stored graph.
                     If the entity was marked as to be deleted, then we're
@@ -197,16 +197,12 @@ class Storer(object):
                     graph_identifier: URIRef = entity.g.identifier
                     for triple in entity.g.triples((entity.res, None, None)):
                         quads.append((*triple, graph_identifier))
-                    stored_g.addN(quads)
-
-            # We must ensure that the graph is correctly stored in our cache
-            if cur_file_path not in already_processed:
-                already_processed[cur_file_path] = stored_g
+                    destination_g.addN(quads)
 
             if store_now:
-                self._store_in_file(stored_g, cur_file_path, context_path)
+                self._store_in_file(destination_g, cur_file_path, context_path)
 
-            return already_processed
+            return destination_g
         except Exception as e:
             self.reperr.add_sentence(f"[1] It was impossible to store the RDF statements in {cur_file_path}. {e}")
 
